"""Calculate the packet overhead to send the given arguments in SNAP Connect"""

import warnings

try:
    import snaplib.SnapConstantsHelper as sch
except ImportError:
    # This moved in newer versions of SNAP Connect
    # Try both locations to support both
    import snappy.SnapConstantsHelper as sch

import snaplib.RpcCodec


def get_dmcast_packet_overhead(func, *args):
    """Return the max packet overhead of the given args for a dmcast packet.

    Right now, we just append 3 integers representing  Group, TTL, and Delay

    Returns:
        The max packet overhead of the given args for a dmcast packet.
    """
    # TODO: Remove this function (or this warning) by version 9.0.0
    warnings.warn("This function is now unused by SNAPconnect Futures itself "
                  "and will be removed in a later major revision. "
                  "See snaplib.RpcCodec.get_dmcast_rpc_bytes_remaining().", DeprecationWarning)

    # Extracted "build a packet to determine its length" logic to snaplib.RpcCodec
    #
    # MAX = Overhead(given func and args) + Remaining(given func and args),
    # so Remaining(given func and args) = MAX - Overhead(given func and args),
    # and Overhead(given func and args) = MAX - Remaining(given func and args)
    #
    # NB: get_*rpc_bytes_remaining acts as if both RPC CRC and Packet CRC are enabled.

    # Also NB: This is overhead for 0 targets, the remaining bytes would be
    #          for targets *and* any additional payload

    return sch.MAX_PKT_SIZE - snaplib.RpcCodec.get_dmcast_rpc_bytes_remaining('', func, *args)


def get_packet_overhead(func, *args):
    """Return the max packet overhead of the given args

    Returns:
        The max packet overhead of the given args
    """
    # TODO: Remove this function (or this warning) by version 9.0.0
    warnings.warn("This function is now unused by SNAPconnect Futures itself "
                  "and will be removed in a later major revision. "
                  "See snaplib.RpcCodec.get_mcast_rpc_bytes_remaining().", DeprecationWarning)

    # Extracted "build a packet to determine its length" logic to snaplib.RpcCodec
    #
    # MAX = Overhead(given func and args) + Remaining(given func and args),
    # so Remaining(given func and args) = MAX - Overhead(given func and args),
    # and Overhead(given func and args) = MAX - Remaining(given func and args)
    #
    # NB: get_mcast_rpc_bytes_remaining acts as if both RPC CRC and Packet CRC are enabled.
    return sch.MAX_PKT_SIZE - snaplib.RpcCodec.get_mcast_rpc_bytes_remaining(func, *args)


def get_max_dmcast_targets(func, *args):
    """Return the maximum number of target addresses that will fit in a dmcast packet

    Args:
        func (str): The RPC function
        args: A variable length list of RPC arguments

    Returns:
        int: The maximum number of target addresses that will fit in a dmcast packet
    """
    try:
        max_targets = snaplib.RpcCodec.get_dmcast_rpc_bytes_remaining('', func, *args) // sch.SNAP2_ADDR_LEN
    except (TypeError, snaplib.RpcCodec.RpcError):
        max_targets = 0

    return max(max_targets, 0)


def get_max_addrs(overhead):
    """Return the maximum number of target addresses that will fit in a packet

    Args:
        overhead (int): The overhead of the packet

    Returns:
        The maximum number of target addresses that will fit in a packet.
    """
    warnings.warn("This function is conservative, use get_max_dmcast_targets "
                  "for more accurate dmcast target counts.", DeprecationWarning)

    # TODO: Remove this function (or this warning) by version 9.0.0
    warnings.warn("This function is now unused by SNAPconnect Futures itself "
                  "and will be removed in a later major revision.", DeprecationWarning)

    # The source of the overhead number may have already accounted for Packet CRC.
    return ((sch.MAX_PKT_SIZE - sch.PKT_CRC_SIZE) - overhead) / sch.SNAP2_ADDR_LEN
