from tornado.testing import AsyncTestCase
from tornado.ioloop import IOLoop


class SnapConnectFuturesAutoTestCase(AsyncTestCase):
    """tornado.testing.AsyncTestCase with the correct override for get_new_ioloop to allow SnapConnectFuturesAuto to be used in test cases.

    Example:

        .. code-block:: python

            from snapconnect_futures.testing import SnapConnectFuturesAutoTestCase
            from tornado.testing import gen_test


            class MyTestCase(SnapConnectFuturesAutoTestCase):

                @gen_test
                def test_that_uses_scfa(self):
                    scf = SnapConnectFuturesAuto()
                    yield scf.open_serial(1, 0)
                    result = yield scf.callback_rpc("0DECAF", "getChannel")
                    self.assertEqual(4, result)
    """

    def __init__(self, *args, **kwargs):
        # Ensure we call the AsyncTestCase __init__ to properly set up the async bits.
        super(SnapConnectFuturesAutoTestCase, self).__init__(*args, **kwargs)

    def get_new_ioloop(self):
        """Return the singleton IOLoop.instance() because that's the instance SNAPconnectFuturesAuto expects to be running."""
        return IOLoop.instance()
